#ifndef _GB18030_H
#define _GB18030_H

#include "fcitx/candidate.h"
#include "fcitx/instance.h"

static void *createInputMethod(FcitxInstance *instance);

static boolean initInputMethod(void *arg);
static INPUT_RETURN_VALUE processInput(void *arg, FcitxKeySym sym,
                                       unsigned int state);
static INPUT_RETURN_VALUE getCandidateWords(void *arg);

// 四字节部分采用GB/T11383——1989未采用的0x30～0x39作为对双字节编码扩充的后缀，
// 编码范围为0x81308130～0xFE39FE39
static int is4ByteCode(FcitxInputState *input);
// GB18030双字节编码最多有16个候选，四字节编码最多有10个候选
static int getMaxCandidateCount(FcitxInputState *input);
// GB18030使用双字节和四字节对汉字进行编码
static int getMaxByteCodeCount(FcitxInputState *input);
// GB18030使用双字节和四字节对汉字进行编码并且使用16进制数字
// 来表示相应对内码，2个16进制符号编码的数字可以用一个字节可以
// 表示对应对数值
static int getMaxRawInputBufferSize(FcitxInputState *input);

typedef struct {
    char buffer[5];
    size_t length;
} InputCodeBuffer;

static void initInputCodeBufferFromInput(InputCodeBuffer *inputCodeBuffer,
                                         FcitxInputState *input);

typedef struct _ImeState ImeState;
typedef void (*ImeStateConvert)(ImeState *state);

typedef struct _ImeState {
    InputCodeBuffer inputCodeBuffer;
    char utf8WordBuffer[UTF8_MAX_LENGTH + 1];
    FcitxInstance *owner;

    ImeStateConvert convert;
} ImeState;

static void imeStateConvert(ImeState *state);

static INPUT_RETURN_VALUE getCandidateWord(void *arg,
                                           FcitxCandidateWord *candidateWord);
static FcitxCandidateWord makeCandidateWord(void *owner, char *word);

#endif